local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:OnInitialize()
	self.updateHandler = CreateFrame("frame")
	self.updateHandler:Hide()
	self.elapsed = 0
	self.updateHandler:SetScript("OnUpdate", function(self, elapsed)
												self = Capitalist
--												self:DebugEntry("OnUpdate")
												self.elapsed = self.elapsed + elapsed
--												self:DebugMisc(self.mode)
												
												if (self.mode == "scanning") then
													self:OnUpdateScanning()
												elseif (self.mode == "posting") then
													self:OnUpdatePosting()
												elseif (self.mode == "componentsearching") then
													self:OnUpdateComponentSearching()
												else
													self:OnUpdateComponentScanning()
												end
--												self:DebugExit("OnUpdate")
											end)
	
	
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("AUCTION_HOUSE_SHOW")
	self:RegisterEvent("AUCTION_OWNED_LIST_UPDATE")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UI_ERROR_MESSAGE")
	self:RegisterEvent("AUCTION_HOUSE_CLOSED")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_CLOSED")
	
	
	if (IsAddOnLoaded("Blizzard_AuctionUI")) then
		self:ADDON_LOADED("ADDON_LOADED", "Blizzard_AuctionUI")
	end
	if (IsAddOnLoaded("Blizzard_TradeSkillUI")) then
		self:ADDON_LOADED("ADDON_LOADED", "Blizzard_TradeSkillUI")
	end

	local defaults = {
			profile = {
				version = 0.11,
			
				margin = 100,
				minMargin = 100*10000,
				maxMargin = 300*10000,
				minProfit = 0*10000,
				showByDefault = false,
				cancelYellow = true,
				cancelBlue = true,
				cancelDuration = true,
				filters = {
					global = {
						Northrend = true,
						BurningCrusade = false,
						Classic = false,
					},
				},
				storage = {},
				debug = {},
			}
		}
	self.db = LibStub("AceDB-3.0"):New("CapitalistDB", defaults, true)
	
	if (not self.db.profile.storage[realm]) then
		self.db.profile.storage[realm] = {}
	end
	
	if (not self.db.profile.storage[realm][UnitName("player")]) then
		self.db.profile.storage[realm][UnitName("player")] = {AH = {}, inventory = {}}
	end
	
	self:DebugMisc("Profile Version: "..self.db.profile.version)
	self:DebugMisc("Current Version: "..defaults.profile.version)
	
	self.db.profile.version = defaults.profile.version
	
	self:ApplyFilters()
	self:BuildList()
	self:BuildOptions()
end

function Capitalist:ADDON_LOADED(event, addon)
	self:DebugEntry("ADDON_LOADED")

	if (addon == "Blizzard_AuctionUI") then
		self:CreateAuctionUI()
		self:CreateComponentUI()
		if (self.db.profile.showByDefault) then
			self:ToggleAuctionUI()
		end
	end
	if (addon == "Blizzard_TradeSkillUI") then
		self.restockButton = CreateFrame("button", "CapitalistRestockButton", TradeSkillFrame, "UIPanelButtonTemplate")
		self.restockButton:SetPoint("TOPRIGHT", TradeSkillFrame, "TOPRIGHT", -25, -2)
		self.restockButton:SetHeight(17)
		self.restockButton:SetWidth(60)
		self.restockButton:SetText("(C)")
		self.restockButton:SetScript("OnClick", function() Capitalist:Restock(CURRENT_TRADESKILL) end)
		self.restockButton:Show()

		self.showStackButton = CreateFrame("button", "CapitalistShowStackButton", TradeSkillFrame, "UIPanelButtonTemplate")
		self.showStackButton:SetPoint("TOPRIGHT", self.restockButton, "TOPLEFT", 0, 0)
		self.showStackButton:SetHeight(17)
		self.showStackButton:SetWidth(30)
		self.showStackButton:SetText("(R)")
		self.showStackButton:SetScript("OnClick", function() Capitalist:ShowRestockStack() end)
		self.showStackButton:Show()
		
		self.TradeSkillFrame_Show_Hook = TradeSkillFrame_Show
		TradeSkillFrame_Show =  function()
									self:DebugMisc(CURRENT_TRADESKILL)
									Capitalist.TradeSkillFrame_Show_Hook()

									-- TODO: Remove after other professions have gotten more attention (like profession filters)
									if (CURRENT_TRADESKILL == L["Enchanting"]) then
										self.restockButton:Show()
										self.showStackButton:Show()
										self:ScanTradeSkillFrame(CURRENT_TRADESKILL)
									else
										self.restockButton:Hide()
										self.showStackButton:Hide()
									end
								end
									
	end

	self:DebugExit("ADDON_LOADED")
end

function Capitalist:AUCTION_HOUSE_CLOSED()
	self:DebugEntry("AUCTION_HOUSE_CLOSED")

	if (self.updateHandler:IsVisible()) then
		self.updateHandler:Hide()
		
		if (self.mode == "scanning") then
			self.auctionUI.lastScanLabel:SetText(L["Last Scan: failed"])
			self.auctionUI.lastScanLabel:Show()
			self.auctionUI.progress:Hide()
			self.auctionUI.progressLabel:Hide()
			self.auctionUI.scanButton:Enable()
			self.validScan = false
		end
	end

	self:DebugExit("AUCTION_HOUSE_CLOSED")
end

function Capitalist:HandleSlashCommand(input)
	self:DebugEntry("HandleSlashCommand")
	self:DebugMisc(input)
	self:DebugMisc(CURRENT_TRADESKILL)

--	if (input == "ProfessionList") then
--		local numSkills = GetNumTradeSkills()
--		self:DebugMisc(numSkills)
--	end

-- TODO: revisit
--	if (input == "restock") then
--		if (not (CURRENT_TRADESKILL == L["Enchanting"] and TradeSkillFrame:IsVisible())) then
--			CastSpellByName(L["Enchanting"])
--		else
--			self:Restock()
--		end
--	else
--		InterfaceOptionsFrame_OpenToCategory("Capitalist")
--	end

	if (input == "top10") then
		-- TODO: enchanting-specific
		self:ReportTop10(L["Enchanting"])
	else
		InterfaceOptionsFrame_OpenToCategory("Capitalist")
	end

	self:DebugExit("HandleSlashCommand")
end
